#include "\lw\include\gpib.h"
#include "\lw\include\formatio.h"
#include "snr488.h"

/*======= KEPCO SNR488 ====================================================*/
/* LabWindows 2.0 Instrument Driver                                        */
/* Original Release: 1/92                                                      */
/* By: Peter Blanco , Kepco Inc.                                                                    */
/* Originally written in C                                                 */
/* Modification History: None                                              */
/*=========================================================================*/

static char cmd[50];
static int snr488_err;
static int bd;
static char *hex_str;
static int init;
static double snr488_vmax[9];
static double snr488_imax[9];
static int snr488_type[9];
static int snr488_range[9];

int snr488_invalid_integer_range();
int snr488_invalid_real_range();
int snr488_device_closed();
int snr488_write_data();

int snr488_init (int addr)
{
    int chn;

    if(init != 1){
        for(chn=0;chn<8;chn++){
            snr488_vmax[chn]=10;
            snr488_imax[chn]=10;
            snr488_type[chn]=0;
            hex_str="0123456789ABCDEF";
        }
        init=1;
    }

/*  default configuration 0-10v voltage , 0-1v current */

    if (snr488_invalid_integer_range (addr, 0, 30, -1) != 0)
        return snr488_err;

/*  If device has not been opened (bd = 0), open it. */

    if (bd <= 0)    {
        CloseInstrDevs("snr488");
        bd = OpenDev ("", "snr488");
        if (bd <= 0)    {
            snr488_err = 220;
            return snr488_err;
        }
    }

/*  Change the primary address of the device    */

    if (ibpad (bd, addr) < 0)   {
        snr488_err = 233;
        return snr488_err;
    }


    return snr488_err;
}

/*=========================================================================*/

int snr488_config(int chn,int range,int type,double volt,double curr)
{
    if (snr488_device_closed())
        return snr488_err;

    if (snr488_invalid_integer_range(chn,0,7,-1) )
        return snr488_err;

    if (snr488_invalid_integer_range(range,0,15,-2) )
        return snr488_err;

    if (snr488_invalid_integer_range(type,0,1,-3) )
        return snr488_err;

    if (snr488_invalid_real_range(volt,0.0,10000.0,-4) )
        return snr488_err;

    if (snr488_invalid_real_range(curr,0.0,1000.0,-5) )
        return snr488_err;

    snr488_vmax[chn] = volt;
    snr488_imax[chn] = curr;
    snr488_type[chn] = type;
    snr488_range[chn] = range;
    snr488_err = 0;
    return 0;
}

int snr488_set(int chn,double volt,double curr)
{
    int d1,d2,d3,range;
    char vout[4],iout[4],dstr[4];

    if (snr488_device_closed())
        return snr488_err;

    if (snr488_invalid_integer_range(chn,0,7,-1) )
        return snr488_err;

    if (snr488_invalid_real_range(volt,-snr488_vmax[chn],snr488_vmax[chn],-2) )
        return snr488_err;

    if (snr488_invalid_real_range(curr,-snr488_imax[chn],snr488_imax[chn],-3) )
        return snr488_err;

    range=snr488_range[chn];

    if(volt<0)
        switch(range){
            case 0:
            case 2:
            case 4:
            case 6:
            case 8:
            case 10:
            case 12:
            case 14:
                range++;
                break;
            case 1:
            case 3:
            case 5:
            case 7:
            case 9:
            case 11:
            case 13:
            case 15:
                range--;
                break;
        }
    if(curr<0)
        switch(range){
            case 0:
            case 1:
            case 2:
            case 3:
            case 8:
            case 9:
            case 10:
            case 11:
                range+=4;
                break;
            case 4:
            case 5:
            case 6:
            case 7:
            case 12:
            case 13:
            case 14:
            case 15:
                range-=4;
                break;
        }
    volt=fabs(volt);
    curr=fabs(curr);
    if(snr488_type[chn]){
        d1=(volt/snr488_vmax[chn])*999;
        d2=(curr/snr488_imax[chn])*999;
        Fmt(vout,"%s<%i[w3p0]",d1);
        Fmt(iout,"%s<%i[w3p0]",d2);
     }

     else{
         volt=(volt/snr488_vmax[chn])*4095;
         d1=volt/256;
         volt-=(d1*256);
         d2=volt/16;
         d3=volt-(d2*16);
         vout[0]=hex_str[d1];
         vout[1]=hex_str[d2];
         vout[2]=hex_str[d3];

         curr=(curr/snr488_imax[chn])*4095;
         d1=curr/256;
         curr-=(d1*256);
         d2=curr/16;
         d3=curr-(d2*16);
         iout[0]=hex_str[d1];
         iout[1]=hex_str[d2];
         iout[2]=hex_str[d3];
    }
    vout[3]=0;
    iout[3]=0;
    Fmt(cmd,"%s<%i=%c%s%s",chn,hex_str[range],vout,iout);
    return snr488_write_data(cmd,9);
}
/*=========================================================================*/
/* This function removes the instrument from the GPIB device table and     */
/* sets bd = 0.                                                            */
/*=========================================================================*/
int snr488_close ()
{
/*  Check for device closed */

    if (snr488_device_closed())
        return snr488_err;

/*  Close the device    */

    if (CloseDev (bd) < 0)
        snr488_err = 221;
    else
        snr488_err = 0;

    bd = 0;
    return snr488_err;
}

/*= UTILITY ROUTINES ======================================================*/

/*=========================================================================*/
/* This function checks an integer to see if it lies between a minimum     */
/* and maximum value.  If the value is out of range, set the global error  */
/* variable to the value err_code.  The return value is equal 0 for        */
/* success and -1 for error.                                               */
/*=========================================================================*/
int snr488_invalid_integer_range (val, min, max, err_code)
int val;
int min;
int max;
int err_code;
{
  if ((val < min) || (val > max)) {
    snr488_err = err_code;
    return -1;
  }
  return 0;
}

/*=========================================================================*/
/* This function checks a real number to see if it lies between a minimum  */
/* and maximum value.  If the value is out of range, set the global error  */
/* variable to the value err_code.  The return value is equal 0 for        */
/* success and -1 for error.                                               */
/*=========================================================================*/
int snr488_invalid_real_range (val, min, max, err_code)
double val;
double min;
double max;
int err_code;
{
  if ((val < min) || (val > max)) {
    snr488_err = err_code;
    return -1;
  }
  return 0;
}

/*=========================================================================*/
/* This function checks to see if the module has been initialized.  If the */
/* device has not been opened, a -1 is returned, 0 otherwise.              */
/*=========================================================================*/
int snr488_device_closed ()
{
    if (bd <= 0)  {
        snr488_err = 232;
        return -1;
    }
    return 0;
}

/*=========================================================================*/
/* This function writes a buffer of data to the GPIB interface.  The       */
/* return value is equal to the global error variable.                     */
/*=========================================================================*/
int snr488_write_data (buf, cnt)
char *buf;
int cnt;
{
    if (ibwrt(bd, buf, (long)cnt) <= 0)
        snr488_err = 230;
    else
        snr488_err = 0;

    return snr488_err;
}


